﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    internal abstract class AbstractMap
    {
        private IDrawningObject _drawningObject = null;

        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;

        public Bitmap CreateMap(int width, int height, IDrawningObject drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }

        private bool CanMove(Direction direction, (float Left, float Right, float Top, float Bottom) position) //Проверка на возможность шага
        {
            
            for (int i = (int)(position.Top / _size_y); i <=(int)((position.Bottom + 25) / _size_y); ++i)
            {
                for (int j = (int)(position.Left / _size_x); j <= (int)((position.Right + 5) / _size_x); ++j)
                {
                    if (i >= 0 && j >= 0 && i < _map.GetLength(0) && j < _map.GetLength(1) && _map[i, j] == _barrier) return false;
                }
            }
            return true;
        }

        public Bitmap MoveObject(Direction direction)
        {
            (float Left, float Right, float Top, float Bottom) position = _drawningObject.GetCurrentPosition();
            if (direction == Direction.Left)
            {
                position.Left -= _drawningObject.Step;
            }
            else if (direction == Direction.Right)
            {
                position.Right += _drawningObject.Step;
            }
            else if (direction == Direction.Up)
            {
                position.Top -= _drawningObject.Step;
            }
            else if (direction == Direction.Down)
            {
                position.Bottom += _drawningObject.Step;
            }

            if (CanMove(direction, position))
            {
                _drawningObject.MoveObject(direction);
            }
            return DrawMapWithObject();
        }

        private bool SetObjectOnMap()
        {
            if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 10);
            int y = _random.Next(0, 10);
            _drawningObject.SetObject(x, y, _width, _height);
            for (int i = (int)(_drawningObject.GetCurrentPosition().Top / _size_y); i <= (int)((_drawningObject.GetCurrentPosition().Bottom + 25) / _size_y); ++i)
            {
                for (int j = (int)(_drawningObject.GetCurrentPosition().Left / _size_x); j <= (int)((_drawningObject.GetCurrentPosition().Right + 5)/ _size_x); ++j)
                {
                    if (_map[i, j] == _barrier) _map[i, j] = _freeRoad;
                }
            }
            return true;
        }
        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (_map[i, j] == _freeRoad)
                    {
                        DrawRoadPart(gr, i, j);
                    }
                    else if (_map[i, j] == _barrier)
                    {
                        DrawBarrierPart(gr, i, j);
                    }
                }
            }
            _drawningObject.DrawningObject(gr);
            return bmp;
        }
        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}
